package com.persagy.adm.diagram.controller;

import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.adm.diagram.core.DataStrategy;
import com.persagy.adm.diagram.entity.DiagramType;
import com.persagy.adm.diagram.frame.BdtpRequest;
import com.persagy.adm.diagram.service.BdtpDataService;
import com.persagy.dmp.common.model.response.CommonResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

@RestController
@CrossOrigin
public class CommonController {

	@Autowired
	private BdtpDataService bdtpDataService;

	@Autowired
	@Qualifier(DataStrategy.implQualifier)
	private DataStrategy dataStrategy;

	@RequestMapping("equipTypes")
	public CommonResult<List<ObjectNode>> equipTypes(){
		return CommonResult.success(bdtpDataService.getEquipTypeList(new BdtpRequest(), null));
	}

	@RequestMapping("relTypes")
	public CommonResult<List<Map<String, Object>>> relTypes(){
		return CommonResult.success(bdtpDataService.getRelTypeTree(new BdtpRequest()));
	}

	@RequestMapping("majSys")
	public CommonResult<List<ObjectNode>> majSys(){
		BdtpRequest req = new BdtpRequest();
		List<ObjectNode> majList = bdtpDataService.getMajorList(req);
		List<ObjectNode> sysList = bdtpDataService.getSystemList(req);

		for(ObjectNode maj : majList){
			ArrayNode childrenSys = maj.putArray("children");
			String majorCode = maj.get("code").asText();
			maj.put("name", maj.get("label").asText());
			for(Iterator<ObjectNode> sysIter = sysList.iterator(); sysIter.hasNext();){
				ObjectNode sys = sysIter.next();
				if(sys.get("majorCode") != null && sys.get("majorCode").asText().equals(majorCode)){
					childrenSys.add(sys);
					sysIter.remove();
				}
			}
		}
		return CommonResult.success(majList);
	}

	@RequestMapping("diagramTypes")
	public CommonResult<List<DiagramType>> diagramTypes(){
		return CommonResult.success(dataStrategy.getDiagramTypes());
	}

}
