package com.persagy.adm.diagram.controller;

import com.persagy.adm.diagram.frame.EditRequest;
import com.persagy.adm.diagram.manage.Folder;
import com.persagy.adm.diagram.manage.TemplateManager;
import com.persagy.adm.diagram.core.model.template.DiagramTemplate;
import com.persagy.dmp.common.model.response.CommonResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@RestController
@CrossOrigin
@RequestMapping("template")
public class TemplateController {

    @Autowired
    private TemplateManager templateManager;

    @RequestMapping("getTemplates")
    public CommonResult<List<Folder>> getTemplates(){
        return CommonResult.success(templateManager.getTemplates());
    }

    @RequestMapping("getTemplatesByType")
    public CommonResult<List<DiagramTemplate>> getTemplatesByType(@RequestParam String diagramType){
        return CommonResult.success(templateManager.getTemplates(diagramType));
    }

    @RequestMapping("newTemplate")
    public CommonResult<DiagramTemplate> newTemplate(@RequestBody Map<String, String> params){
        return CommonResult.success(templateManager.createTemplate(params.get("name"), params.get("type")));
    }

    @RequestMapping("delTemplate")
    public CommonResult<Boolean> delTemplate(@RequestBody Map<String, String> params){
        return CommonResult.success(templateManager.deleteTemplate(params.get("id")));
    }

    @RequestMapping("addCon")
    public CommonResult<DiagramTemplate> addCon(@RequestBody EditRequest params){
        return CommonResult.success(templateManager.addCon(params.getNewCompId(), params.getCurrentCompId(), params.getTemplateId()));
    }

    @RequestMapping("delCon")
    public CommonResult<DiagramTemplate> delCon(@RequestBody EditRequest params){
        return CommonResult.success(templateManager.delCon(params.getCurrentCompId(), params.getTemplateId()));
    }

    @RequestMapping("moveCon")
    public CommonResult<DiagramTemplate> moveCon(@RequestBody EditRequest params){
        return CommonResult.success(templateManager.moveCon(params.getCurrentCompId(), params.getTemplateId(), params.getMoveOffset()));
    }

    @RequestMapping("addMainPipe")
    public CommonResult<DiagramTemplate> addMainPipe(@RequestBody EditRequest params){
        return CommonResult.success(templateManager.addMainPipe(params.getLines(), params.getTemplateId()));
    }

    @RequestMapping("delLine")
    public CommonResult<DiagramTemplate> delLine(@RequestBody EditRequest params){
        return CommonResult.success(templateManager.delMainPipe(params.getCurrentCompId(), params.getTemplateId()));
    }

    @RequestMapping("modifyCon")
    public CommonResult<DiagramTemplate> modifyCon(@RequestBody EditRequest params){
        return CommonResult.success(templateManager.modifyCon(params.getTemplatePropsData(), params.getCurrentCompId(), params.getTemplateId()));
    }

    @RequestMapping("modifyMainPipe")
    public CommonResult<DiagramTemplate> modifyMainPipe(@RequestBody EditRequest params){
        return CommonResult.success(templateManager.modifyMainPipe(params.getTemplatePropsData(), params.getCurrentCompId(), params.getTemplateId()));
    }

    /**
     * 编辑设备过滤条件
     */
    @RequestMapping("modifyFilter")
    public CommonResult<DiagramTemplate> modifyFilter(@RequestBody EditRequest params){
        return CommonResult.success(templateManager.modifyFilter(params.getDataFilter(), params.getCurrentCompId(), params.getTemplateId()));
    }

    /**
     * 编辑设备打包配置
     */
    @RequestMapping("modifyPack")
    public CommonResult<DiagramTemplate> modifyPack(@RequestBody EditRequest params){
        return CommonResult.success(templateManager.modifyPack(params.getEquipPack(), params.getCurrentCompId(), params.getTemplateId()));
    }

    /**
     * 编辑容器的动态组配置
     */
    @RequestMapping("modifyDynGroup")
    public CommonResult<DiagramTemplate> modifyDynGroup(@RequestBody EditRequest params){
        return CommonResult.success(templateManager.modifyDynGroup(params.getDynGroup(), params.getCurrentCompId(), params.getTemplateId()));
    }

}